/*  Author: Dr. Liping Di, Hughes STX Corporation

 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its 
 documentation for any purpose without fee is hereby granted, provided 
 that the above copyright notice appear in all copies and that both that 
 copyright notice and this permission notice appear in supporting 
 documentation.

*/

#ifndef HDFUTIL_H
#define HDFUTIL_H


/* function prototype */

void hdf_print(FILE *fp,         /* In: ASCII file pointer */ 
              int32 num_type,   /* In: The HDF type of values to be printed */ 
              int count,        /* In: The number of values in the value buffer */
              void *value);      /* In: the value buffer */

int hdf_minmax(void *buffer,      /* In:  Buffer containing vdata records,
                                          buffer[0] is the start position 
                                          of the field interested */
           int16 nt,              /* In:  The data type for the field */
           int n_elements,        /* In:  Number of (elements) records in 
                                          the buffer */
           int nbytes,            /* In:  number of bytes per vdata records */
           double *vmin,          /* Out: The minimum value of the data */
           double *vmax);         /* Out: The maximum value of the data */

int hdf_scaling(void *buffer,     /* In:  Buffer containing vdata records,
                                          buffer[0] is the start position 
                                          of the field interested */
                int nt,           /* In:  The HDF data type for the field */
                int n_elements,   /* In:  Number of elements (records) in buffer */
                int nbytes,       /* In:  Number of bytes for one record */
                int n_levels,     /* In:  Number of data levels after scaling.
                                          The data will be scaled to the range
                                          from 0 to n_levels-1. */
                double vmin,      /* In:  The minimum value of the field */
                double vmax,      /* In:  The maximum value of the field */
                unsigned short *out);  /* Out: The buffer containing the scaled
                                              value. The buffer should be
                                              supplied by the calling routine
                                              and at least n_elements long */

int hdf_mmscaling(void *buffer,   /* In:  Buffer containing HDF data with
                                          same data type */ 
                int nt,           /* In:  The HDF data type */
                int n_elements,   /* In:  Number of data elements in buffer */
                int n_levels,     /* In:  Number of data levels after scaling.
                                          The data will be scaled to the range
                                          from 0 to n_levels-1. */
                unsigned short *out,  /* Out: The buffer containing the scaled
                                              value. The buffer should be
                                              supplied by the calling routine
                                              and at least n_elements long */
                double *vmin,         /* Out: The minimum value calculated */
                double *vmax);         /* Out: The maximum value calculated */

#endif
