/****************************************************************************
 * NCSA HDF                                                                 *
 * Software Development Group                                               *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * COPYING file.                                                            *
 *                                                                          *
 ****************************************************************************/
#ifdef RCSID
static char RcsId[] = "@(#)sdsdump.c,v 1.10 1996/04/15 17:58:29 georgev Exp";
#endif

#include <math.h>
#include <ctype.h>
#include "sdb.h"
#include "sdsdump.h"
#include "html_util.h"

/* printing functions copied from vshow.c and used by sdsdumpfull(). 
*/
static int32 cn = 0;

int32 fmtbyte(unsigned char *x, FILE *ofp)
{
    cn += fprintf(ofp,"%02x ",*x);
    return SUCCEED;
}

int32 fmtint8(VOIDP x, FILE *ofp)
{
    cn += fprintf(ofp, "%d", (int) *((signed char *)x));
    return SUCCEED;
}

int32 fmtuint8(VOIDP x, FILE *ofp)
{
    cn += fprintf(ofp, "%u", (unsigned) *((unsigned char *)x));
    return SUCCEED;
}

int32 fmtint16(VOIDP x, FILE *ofp)
{
  short s = 0;
  HDmemcpy(&s, x, sizeof(int16));
  cn += fprintf(ofp, "%d", (int)s);
  return SUCCEED;
}

int32 fmtuint16(VOIDP x, FILE *ofp)
{
  unsigned short s = 0;
  HDmemcpy(&s, x, sizeof(int16));
  cn += fprintf(ofp, "%u", (unsigned)s);
  return SUCCEED;
}

int32 fmtchar(VOIDP x, FILE *ofp)
{
    if (isprint(*(unsigned char *)x))   
      {
        putc(*((char *)x), ofp);
        cn++;
      }
    else   
      {
        putc('\\', ofp);
        cn++;
        cn += fprintf(ofp, "%03o", *((uchar8 *)x));
      }
    return SUCCEED;
}

int32 fmtuchar8(VOIDP x, FILE *ofp)
{
    cn++;
    putc('\\', ofp);
    cn++;
    fprintf(ofp, "%o", *((uchar8 *)x));
    return SUCCEED;
}

int32 fmtint(VOIDP x, FILE *ofp)
{
  int i = 0;
  HDmemcpy(&i, x, sizeof(int32));
  cn += fprintf(ofp, "%d", (int)i);
  return SUCCEED;
}

int32 fmtfloat32(VOIDP x, FILE *ofp)
{
  float32 f = 0, epsi = 1.0e-20;
  HDmemcpy(&f, x, sizeof(float32));
  if (fabs(f - FILL_FLOAT) <= epsi)
    cn += fprintf(ofp, "FloatInf ");
  else 
    cn += fprintf(ofp, "%f", f);
  return SUCCEED;
}

int32 fmtint32(VOIDP x, FILE *ofp)
{
  long l = 0;
  HDmemcpy(&l, x, sizeof(int32));
  cn += fprintf(ofp, "%ld", (long)l);
  return SUCCEED;
}

int32 fmtuint32(VOIDP x, FILE *ofp)
{
  uint32 l = 0;
  HDmemcpy(&l, x, sizeof(int32));
  cn += fprintf(ofp, "%lu", (unsigned long)l);
  return SUCCEED;
}

int32 fmtshort(VOIDP x, FILE *ofp)
{
  short s = 0;
  HDmemcpy(&s, x, sizeof(int16));
  cn += fprintf(ofp, "%d", s);
  return SUCCEED;
}

#define FLOAT64_EPSILON ((float64)1.0e-20)
int32 fmtfloat64(VOIDP x, FILE *ofp)
{
  double d = 0, epsi = 1.0e-20;

  HDmemcpy(&d, x, sizeof(float64));
  if (fabs(d - FILL_DOUBLE) <= FLOAT64_EPSILON)
    cn += fprintf(ofp, "DoubleInf ");
  else 
    cn += fprintf(ofp, "%f", d);
  return SUCCEED;
}

int32 dumpfull(int32 nt,int32 cnt,VOIDP databuf,intn indent,FILE *ofp)
{
    intn i; 
    VOIDP b; 
    int32 (*fmtfunct)(VOIDP x, FILE *ofp)=NULL;
    int32 off;
    
    switch (nt)  
      {
       case DFNT_CHAR:
            fmtfunct = fmtchar;
            break;
       case DFNT_UCHAR:
            fmtfunct = fmtuchar8;
            break;
       case DFNT_UINT8:
            fmtfunct = fmtuint8;
            break;
       case DFNT_INT8:
            fmtfunct = fmtint8;
            break;
       case DFNT_UINT16:
            fmtfunct = fmtuint16;
            break;
       case DFNT_INT16:
            fmtfunct = fmtint16;
            break;
       case DFNT_UINT32:
            fmtfunct = fmtuint32;
            break;
       case DFNT_INT32:
            fmtfunct = fmtint32;
            break;
       case DFNT_FLOAT32:
            fmtfunct = fmtfloat32;
            break;
       case DFNT_FLOAT64:
            fmtfunct = fmtfloat64;
            break;
       default:
            fprintf(ofp, "HDP does not support type [%i] \n", (int)nt);
            break;
      }
    b = databuf;
    off = DFKNTsize(nt | DFNT_NATIVE);
    cn = indent;
    for (i=0; i<cnt; i++)   
      {
        fmtfunct(b, ofp);
        b = (char *)b+ off;
        if (nt != DFNT_CHAR)   
          {
            putc(' ', ofp);
            cn++;
          }
		if(indent > 0){
        	if (cn > 65)   
          	{
            	putc('\n', ofp);
            	for (cn=0; cn<indent; cn++) putc(' ', ofp);
          	}
        }
      }
    return SUCCEED;
}	    


int32 sdsdumpfull(int32 sds_id, int32 rank, int32 dimsizes[], int32 nt, intn indent, FILE *fp, int32 *start, int32 *stride, int32 *edge, int32 *left, lvar_st *l_vars)
{   
    /* "rank" is the number of dimensions and 
       "dimsizes[i]" is size of dimension "i".*/
/* note: Modified by lpd for the case of indent = 0 at which this function will dump 
   the data in ASCII text form */

    int32 j, ret;
    VOIDP buf; 
    int32 numtype, eltsz, read_nelts, num_elts, temp;
    /* int32 *left, *start, *edge; */
    int32 done;    /* number of rows we have done */

    if (indent>65)  
      { /* This block is probably not necessary. */
         gateway_err(l_vars->hfp,"sds_dumpfull: Bad indentation ",0);
         return FAIL;
      } 
  
    /* Compute the number of the bytes for each value. */
    numtype = nt & DFNT_MASK;
    eltsz = DFKNTsize(numtype | DFNT_NATIVE); 

    /* read_nelts = dimsizes[rank-1]; */
    read_nelts = left[rank-1];
    if ((buf=(VOIDP)HDgetspace(read_nelts*eltsz)) == NULL)
      {
         gateway_err(l_vars->hfp,"sds_dumpfull: failed to allocate space ",0);
         return FAIL;
      }


    if ((temp=(read_nelts%stride[rank-1]))!=0)
       num_elts = read_nelts/stride[rank-1] + 1;
    else
       num_elts = read_nelts/stride[rank-1];
    
    edge[rank-1]=num_elts;  /* edge = number of values to be retrieved. lpd */
    
    if (rank==1) 
      {   /* If there is only one dimension, then dump the data
	     and the job is done. */
        if ((ret = SDreaddata(sds_id, start, stride, edge, buf)) == FAIL)
          {
            gateway_err(l_vars->hfp,"sds_dumpfull: failed to read SDS ",0);
            return FAIL;
          }
 
        if ((ret = dumpfull(numtype, num_elts, buf, indent, fp)) == FAIL)
         {
           gateway_err(l_vars->hfp,"sds_dumpfull: failed to dump data ",0);
           return FAIL;
         }
 
        fprintf(fp,"\n"); 
        cn=0;
        done = 1;
     }

   done = 0; 
    while (!done && (rank-2)>=0)  
      {  /* Dump a row each time modify left[] accordingly. */
        /* In each execution of "SDreaddata", only one unit (starting from
	   "start[j]) from each of the first (rank-1) dimensions is read in.
	   and all the elements in the last dimension are all read in at 
	   once. */
        if ((ret = SDreaddata(sds_id, start, stride, edge, buf)) == FAIL) 
          {
            gateway_err(l_vars->hfp,"sds_dumpfull: failed to read SDS ",0);
            return FAIL;
          }

        
	if ((ret = dumpfull(numtype, num_elts, buf, indent, fp)) == FAIL)
          {
            gateway_err(l_vars->hfp,"sds_dumpfull: failed to dump data ",0);
            return FAIL;
          }
    if(indent > 0){
	fprintf(fp,"<P>");
        for (cn=0; cn<indent; cn++) 
          fprintf(fp, " ");
	}
	else
		fprintf(fp, "\n");
		
        for (j = rank-2; j >= 0; j--)  
          { /* Examine each dimension. */
	    /* Don't have to consider the last dimension since all the 
               elements in that dimension are to be read in at once. */
            left[j] = left[j] - stride[j];
            if (left[j] > 0) 
              { /* Stay in the same dimension. */
                /* We have "--left[j]" because the first call
                   to SDreaddata already dealt with the first
                   unit of each of the first (rank-1) 
                   dimensions. */
                /*start[j]++; */ /* Advance to the next unit of the current 
                  dimension. */

                start[j] = start[j] + stride[j];
/*
                  if ((temp=(read_nelts%stride[j]))!=0)
                     num_elts = read_nelts/stride[j] + 1;
                  else
                     num_elts = read_nelts/stride[j];
 */                 
                break;
              }
            else     
              {              /* borrow one from j-1 dimension  */
                left[j] = dimsizes[j];
                start[j] = 0;
                if (j==0) 
                  done=1; /* At this point, all the units in the last dimension
                             have been displayed. So, just stop. */
                if (j==rank-2)  
                  { 
                  	if(indent > 0){
                    	fprintf(fp, "<P>");
                    	for (cn=0; cn<indent; cn++) putc(' ', fp);
                    }
                  }
              }
          }    /* for j */
      }  /* while   */

    HDfreespace((VOIDP)buf);
	if(indent > 0)
    	fprintf(fp, "\n");
    
    /* printf("Stupid!!!!!!"); exit(-1); */
   
    return SUCCEED;
} /* sdsdumpfull */
/* ------------------------------------- */

