
/*--------------------------------------------------------------------
 NAME
       vd_subset
 DESCRIPTION
       subsetting, downloading, and visualizing a vdata
 RETURNS
       file name for the subseted data.
 AUTHOR
       Dr. Liping Di, Hughes STX.  
 COMMENTS
   
 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its 
 documentation for any purpose without fee is hereby granted, provided 
 that the above copyright notice appear in all copies and that both that 
 copyright notice and this permission notice appear in supporting 
 documentation.

---------------------------------------------------------------------*/
#include <ctype.h>
#include "sdb.h"
#include "sdb_util.h"
#include "html_util.h"
#include "vdsub.h"
#include "plotgif.h"
#include "hdfutil.h"

char *vd_subset(char *fname,       /* file name */
      int32 t,                     /* tag */
      int32 r,                     /* ref */
      lvar_st *l_vars, mime_content_type *mime_type)
{
    int32 i,j;
    int32 fid = FAIL;
    int32 fidtmp;
    int32 vd_idtmp;
    int32 sd_idtmp;
    int32 sd_id = FAIL;
    int32 vd_id = FAIL;
    char   name[VSNAMELENMAX + 1];  /* Vdata name */
    char   class[VSNAMELENMAX + 1]; /* Vdata class */
    char   fields[(FIELDNAMELENMAX + 1) * VSFIELDMAX]; /* Vdata field names*/
    int32  types[VSFIELDMAX];
    int32  locations[VSFIELDMAX+2];
    int32  orders[VSFIELDMAX];
    int32  n_records;
    int32  sz;        /* size of Vdata */
    int32  cnt;       
    FILE   *h_fp;        /* HTML for error message */
    FILE   *fh_fp;    /* file for the retrieved data */
    int ret_value = SUCCEED;
    char *tmp = (char *)tempnam(NULL, "html");                
    char *ftmp = NULL;
    char *buffer;
    char tmp_buf[32768];
    char *name_only;
    int32 ndattr, type;
    int nvars;
    
    char x_name[VSNAMELENMAX+1], y_name[VSNAMELENMAX + 1], plot_title[2*VSNAMELENMAX + 7];
    void *x_value=NULL, *y_value=NULL;
    int32 crec, xtype, ytype;
    int16 xsize, ysize, xloc, yloc;
    char *xpt, *ypt;

    
    ENTER(2,"vd_subset");
    name_only = strrchr(fname, '/');
    if(name_only == NULL)
         name_only = fname;
    else 
         name_only++;
    
    DBUG_PRINT(1,(LOGF,"vd_subset: lets open file\n"));
    
    /* open temporary file */
    if ((h_fp = fopen(tmp, "w")) == NULL)
        gateway_err(stdout,"vd_subset: Problem opening temproray file",1);
              
    /* Write MIME header response */
    *mime_type = TEXT_HTML;
    if (write_html_header(h_fp, *mime_type,l_vars) == FAIL){
        fclose(h_fp);
        remove(tmp);
        gateway_err(stdout,"vd_subset: writing HTML header",1);
    }

    /* open file */
    if ((fid = Hopen(fname, DFACC_RDONLY, 0)) == FAIL){
          gateway_err(h_fp,"vd_subset: Problem opening HDF file",0);
          fclose(h_fp);
          return tmp;
    }

    /* start Vxx inteface */
    if ((Vstart(fid)) == FAIL){
          gateway_err(h_fp,"vd_subset: failed to start Vset interface\n",0);
          Hclose(fid);
          fclose(h_fp);
          return tmp;
    }

    /* attach to Vdata */
    if ((vd_id = VSattach(fid, r, "r")) == FAIL){
          gateway_err(h_fp,"vd_subset: failed to attach to Vdata\n",0);
          Vend(fid);
          Hclose(fid);
          fclose(h_fp);
          return tmp;
    }

    /* special treatment for xy plot */
    if( l_vars->view_type == V_IMAGE ){
        l_vars->nfields = l_vars->nyfields + 1;    /* x + y fields */
        for (i = 0; i < l_vars->nyfields; i++)
            l_vars->field_indices[i+1] = l_vars->yfield_indices[i];
        if(l_vars->nfields == 2 && l_vars->field_indices[1] == l_vars->field_indices[0])
            l_vars->nfields = 1;
    }

    VSinquire(vd_id, &n_records, NULL, fields, &sz, name);
    if(l_vars->vh_start_rec < 1)l_vars->vh_start_rec = 1;
    if(l_vars->vh_end_rec > n_records)l_vars->vh_end_rec = n_records;
    if(l_vars->vh_skip_rec < 1) l_vars->vh_skip_rec = 1;
    fields[0]='\0';
    for(i=0; i<l_vars->nfields; i++){
         if(i != 0)strcat(fields, ",");
         strcat(fields, VFfieldname(vd_id, l_vars->field_indices[i]-1));
    }
    
    /* retrieve the order and data type, and locations */
    locations[0]=0;
    for(i=0; i<l_vars->nfields; i++){
         orders[i]=VFfieldorder(vd_id, l_vars->field_indices[i]-1);     
         locations[i+1]=VFfieldisize(vd_id, l_vars->field_indices[i]-1);
         locations[i+1]=locations[i] + locations[i+1];
         types[i]=VFfieldtype(vd_id, l_vars->field_indices[i]-1);
    }
    sz = VSsizeof(vd_id, fields);
    if(sz != locations[l_vars->nfields])
          DBUG_PRINT(1,(LOGF, "vd_subset: sz=%d locations_size=%d\n",
                          sz, locations[l_vars->nfields]));
    
    DBUG_PRINT(1, (LOGF, "vd_subset:fields=%s\n", fields));
    
    /* set the retrieve fields */
    VSsetfields(vd_id, fields);
    buffer = (char *)malloc(sz);
    if(buffer == NULL){
          VSdetach(vd_id);
          gateway_err(h_fp,"vd_subset: failed to allocate a temporary memory\n",0);          
          Vend(fid);
          Hclose(fid);
          fclose(h_fp);
          return tmp;
    }
          
    switch(l_vars->view_type){
    
    case V_IMAGE:
       if(l_vars->nfields == 1 && (types[0] == DFNT_CHAR8 || types[0] == DFNT_UCHAR8)){
              gateway_err(h_fp, "vd_subset: you have to select a numerical field for plot\n", 1);
              fclose(h_fp);
              break;
         }

         /* find x variable, if both x and y is same variable, plot record id verse y value */
         if(l_vars->nfields == 1){
               xloc=-1;
               yloc=locations[0];
               strcpy(x_name, "Record ID");
               xtype =  DFNT_INT32;
               ytype = types[0];
               xsize = 4;
               ysize = locations[1] - locations[0];
               strcpy(y_name, VFfieldname(vd_id, l_vars->field_indices[0]-1));
          }
          else {
               xloc  = locations[0];
               xtype = types[0];
               xsize = locations[1] - locations[0];
               strcpy(x_name, VFfieldname(vd_id, l_vars->field_indices[0]-1));
               yloc  = locations[1];
               ytype = types[1];
               ysize = locations[l_vars->nfields] - locations[1];
               strcpy(y_name, VFfieldname(vd_id, l_vars->field_indices[1]-1));
          }
          
          /* find number of variables to plot */
          nvars=l_vars->nyfields;

          /* make the plot axis name */
          if(l_vars->x_axis_name){
              i = strlen(l_vars->x_axis_name);
              for(j=0; j<i; j++)
                   if(isalnum(l_vars->x_axis_name[j]))break;
              if(j != i){
                   strncpy(x_name, l_vars->x_axis_name,VSNAMELENMAX);
                   x_name[VSNAMELENMAX] = '\0';
              }
          }

          if(l_vars->y_axis_name){
              i = strlen(l_vars->y_axis_name);
              for(j=0; j<i; j++)
                   if(isalnum(l_vars->y_axis_name[j]))break;
              if(j != i){
                   strncpy(y_name, l_vars->y_axis_name,VSNAMELENMAX);
                   y_name[VSNAMELENMAX] = '\0';
              }
          }
 
          j=-1; 
          if(l_vars->xyplot_name){
               i=strlen(l_vars->xyplot_name);
               for(j=0;j<i; j++)
                   if(isalnum(l_vars->xyplot_name[j]))break;
               if(j==i)
                   j= -1;
               else
                   strcpy(plot_title, l_vars->xyplot_name);
          }
          if(j == -1){
               strcpy(plot_title, x_name);
               strcat(plot_title, " .vs. ");
               strcat(plot_title, y_name);
          }

          /* allocating the memory */
          n_records = (l_vars->vh_end_rec - l_vars->vh_start_rec)/ l_vars->vh_skip_rec +1;
          xpt = x_value = (double *)malloc ((n_records * xsize/sizeof(double) +1 )*sizeof(double));
          ypt = y_value = (double *)malloc ((n_records * ysize/sizeof(double) +1 )*sizeof(double));

          if(x_value == NULL  || y_value == NULL){
                if(x_value != NULL)free(x_value);
                if(y_value != NULL)free(y_value);
                gateway_err(h_fp,"vd_subset: failed to create temporary gif file\n", 1);
                fclose(h_fp);
                break;
          }

          /* read the data */
          crec = 0;
          for(i=l_vars->vh_start_rec; i<= l_vars->vh_end_rec; i=i + l_vars->vh_skip_rec){
                VSseek(vd_id, i-1);
                VSread(vd_id, (uint8 *)buffer, 1, FULL_INTERLACE);
                if(xloc == -1)
                     memcpy(xpt, &crec, 4);
                else
                     memcpy(xpt, buffer+ xloc, xsize);
                memcpy(ypt, buffer+ yloc, ysize);
                xpt += xsize;
                ypt += ysize;
                crec ++;
          }
          for(i=0; i<l_vars->nfields+1; i++)
              DBUG_PRINT(3,(LOGF, "location[%d]=%d\n", i, locations[i]));

          /* preparing the plotting */
          locations[1] = 0;
          if( xloc == -1 ) types[1] = types[0];

          for ( i=1; i<nvars; i++)
              locations[i+1] -= yloc;

          ftmp = (char *)tempnam(NULL, "gif"); 
          if(!plotgif(ftmp, B_XSIZE, B_YSIZE, x_value, y_value, l_vars->xyplot_style, 
                      xtype, &types[1], &locations[1], xsize, ysize, nvars, 
                      l_vars->scaling, n_records, x_name, y_name, plot_title, fields )){

              sprintf(tmp_buf, "vd_subset:plot failed ->%s\n", plot_title);
              gateway_err(h_fp, tmp_buf, 1);
              fclose(h_fp);
              free(ftmp);
              free(x_value);
              free(y_value);
              break;
         }
         free(x_value);
         free(y_value);
         fclose(h_fp);
         remove(tmp);
         free(tmp);
         tmp = ftmp;
        /* set out the header info */
         *mime_type = IMAGE_GIF;
         printf("Content-type: image/gif%c%c",10,10);
#ifdef WIN32
		setoutput_binarymode();
#endif /* WIN32 */
         break;
         
     case V_BINARY_FILE:
        ftmp=(char *)tempnam(NULL, "bin");
        fh_fp=fopen(ftmp, "wb");
        if(fh_fp == NULL){
             gateway_err(h_fp, "vd_subset: failed to open a temporary file\n", 0);
             fclose(h_fp);
             break;
        }        
        
        /* write the data */
        for(i=l_vars->vh_start_rec; i<= l_vars->vh_end_rec; i=i + l_vars->vh_skip_rec){
             VSseek(vd_id, i-1);
             VSread(vd_id, (uint8 *)buffer, 1, FULL_INTERLACE);
             fwrite(buffer, 1, sz, fh_fp);
        }

        fclose(fh_fp); 
        *mime_type = APP_BIN;
        printf("Content-type: application/octet-stream;name=\"test.bin\"%c",10);
        printf("File-name: test.bin%c%c",10,10);
        fclose(h_fp);
        remove(tmp);
        free(tmp);
		tmp=ftmp;
#ifdef WIN32
		setoutput_binarymode();
#endif /* WIN32 */
        break;
                                                         
    case V_HDF_FILE:
        /* open the temporary HDF file */
        ftmp = (char *)tempnam(NULL, "hdf");
        fidtmp = Hopen(ftmp, DFACC_CREATE, 0);
        if(fidtmp == NULL){
             gateway_err(h_fp, "vd_subset: failed to open a temporary HDF file\n",0);
             fclose(h_fp);
             break;
        }
        
        if(Vstart(fidtmp) == FAIL){
             gateway_err(h_fp, "vd_subset: failed to start V interface for the temporary file\n", 0);
             Hclose(fidtmp);
             remove(ftmp);
             fclose(h_fp);
             break;
        }
        
        if((vd_idtmp=VSattach(fidtmp, -1, "w")) == FAIL){
             gateway_err(h_fp, "vd_subset: failed to attach a vdata in the temporary HDF file\n", 0);
             Vend(fidtmp);
             Hclose(fidtmp);
             remove(ftmp);
             fclose(h_fp);
             break;
        }
            
        /* define the fields */
        for(i=0; i<l_vars->nfields; i++)
             VSfdefine(vd_idtmp,VFfieldname(vd_id, l_vars->field_indices[i]-1), types[i], orders[i]);
          
        VSsetfields(vd_idtmp, fields);
        VSsetname(vd_idtmp, name);
        class[0]='\0';
        VSgetclass(vd_id, class);
        VSsetclass(vd_idtmp, class);
          
        /* retrieve the data */
        for(i=l_vars->vh_start_rec; i<=l_vars->vh_end_rec; i=i+l_vars->vh_skip_rec){
             VSseek(vd_id, i-1);
             VSread(vd_id, (uint8 *)buffer, 1, FULL_INTERLACE);
             VSwrite(vd_idtmp,(uint8 *)buffer, 1, FULL_INTERLACE);
        }
        VSdetach(vd_idtmp);
        Vend(fidtmp);
        Hclose(fidtmp);
        
        /* set the attributes */
        if((sd_idtmp=SDstart(ftmp, DFACC_RDWR)) != FAIL){
            sprintf(tmp_buf, "This HDF file, created by DAAC-in-a-Box,\nis a subset of original HDF file: %s", name_only); 
            SDsetattr(sd_idtmp, "DIB data source", DFNT_CHAR8, strlen(tmp_buf), tmp_buf);
            sprintf(tmp_buf, "The Vdata \"%s\" is a subset of the same Vdata\nin the original HDF file: %s.\n" 
                                      "Except for the subset start, skip, and end, all other\n"
                                      "attributes are directly copied from the original file", name, name_only);  
            SDsetattr(sd_idtmp, "Subsetting_description", DFNT_CHAR8, strlen(tmp_buf), tmp_buf);
            cnt=l_vars->vh_start_rec;
            SDsetattr(sd_idtmp, "subset start", DFNT_INT32, 1, &cnt);
            cnt=l_vars->vh_skip_rec;
            SDsetattr(sd_idtmp, "subset skip", DFNT_INT32, 1, &cnt);
            cnt=l_vars->vh_end_rec;
            SDsetattr(sd_idtmp, "subset end", DFNT_INT32, 1, &cnt);
            
            if((sd_id=SDstart(fname, DFACC_RDONLY))!= FAIL){
                SDfileinfo(sd_id, &cnt, &ndattr);   
                 
                /* copy all global attributes */
                for(j = 0; j < ndattr; j++){
                         SDattrinfo(sd_id, j, name, &type, &cnt);
                         SDreadattr(sd_id, j, tmp_buf);
                         SDsetattr(sd_idtmp, name, type, cnt, tmp_buf);
                }
                SDend(sd_id);
            }
            SDend(sd_idtmp);
        }
                
        fclose(h_fp);
        remove(tmp);
        *mime_type = APP_HDF;
        printf("Content-type: application/x-hdf%c",10);
        printf("Name: test.hdf%c%c",10,10);
        free(tmp);
		tmp=ftmp;
#ifdef WIN32
		setoutput_binarymode();
#endif /* WIN32 */
        break;
                            
     case V_ASC_FILE:
        ftmp=(char *)tempnam(NULL, "ASC");
        fh_fp=fopen(ftmp, "w");
        if(fh_fp == NULL){
             gateway_err(h_fp, "vd_subset: failed to open a temporary file\n", 0);
             fclose(h_fp);
             break;
        }        
        
        for(i=l_vars->vh_start_rec; i<= l_vars->vh_end_rec; i=i + l_vars->vh_skip_rec){
             VSseek(vd_id, i-1);
             VSread(vd_id, (uint8 *)buffer, 1, FULL_INTERLACE);
             for(j=0; j<l_vars->nfields; j++){
                  if(j != 0)fprintf(fh_fp, " ");
                  hdf_print(fh_fp, types[j], orders[j], buffer+locations[j]);
             }
             fprintf(fh_fp, "\n");                  
        }
        fclose(fh_fp); 
        *mime_type = APP_TXT;
        printf("Content-type: application/plain-text;name=\"test.ASC\"%c",10);
        printf("File-name: test.ASC%c%c",10,10);
        fclose(h_fp);
        remove(tmp);
        tmp=ftmp;
        break;
        
    default:
        gateway_err(h_fp, "vd_subset: Unknown view type\n", 0);
        fclose(h_fp);
        break;
    }
    free(buffer);
    VSdetach(vd_id);
    Vend(fid);
    Hclose(fid);
    return(tmp);
}

